@@ -20,7 +20,7 @@ Every task is made easy with tons of ``help`` & ``debug`` commands!
 __title__ = 'geocoder'
 __author__ = 'Denis Carriere'
 __author_email__ = 'carriere.denis@gmail.com'
-__version__ = '0.9.0'
+__version__ = '1.0.1'
 __license__ = 'MIT'
 __copyright__ = 'Copyright (c) 2013-2015 Denis Carriere'
 
@@ -29,7 +29,7 @@ from .api import get, yahoo, bing, geonames, google, mapquest
 from .api import nokia, osm, tomtom, geolytica, arcgis, opencage
 
 # EXTRAS
-from .api import ip, canadapost
+from .api import timezone, elevation, ip, canadapost, reverse
 
 # CLI
 from cli import cli
