@@ -1,27 +1,78 @@
 #!/usr/bin/python
 # coding: utf8
 
-from __future__ import print_function
 import requests
 import sys
 
 
 class Base(object):
-    _base_parameter  = [':param ``location``: Your search location you want geocoded.']
-    _base_reference = ['[GitHub Repo](https://github.com/DenisCarriere/geocoder)',
-                       '[GitHub Wiki](https://github.com/DenisCarriere/geocoder/wiki)']
     _exclude = ['parse', 'json', 'url', 'attributes', 'help', 'debug', 'short_name',
-                'api', 'description', 'content', 'params', 'status_code', 'headers',
-                'status_description', 'api_key', 'ok', 'key', 'id', 'x', 'y', 'latlng',
-                'bbox', 'geometry', 'wkt']
-    _example = []
-    _timeout = 5.0
-    attributes = []
+                'api', 'content', 'params', 'status_code', 'street_number', 'method',
+                'api_key', 'ok', 'key', 'id', 'x', 'y', 'latlng', 'headers', 'timeout',
+                'bbox', 'geometry', 'wkt','locality', 'province','rate_limited_get', 'osm',]
+    _attributes = []
+    error = None
+    status_code = None
     headers = {}
-    error = ''
+    params = {}
+    housenumber = ''
+    address = ''
+    lat = ''
+    lng = ''
+    street = ''
+    city = ''
+    state = ''
+    postal = ''
+    country = ''
+    population = ''
 
     def __repr__(self):
-        return "<[{0}] {1} [{2}]>".format(self.status, self.provider, self.address)
+        return "<[{0}] {1} - {2} [{3}]>".format(
+            self.status, 
+            self.provider.title(), 
+            self.method.title(), 
+            self.address
+        )
+    
+    @staticmethod
+    def rate_limited_get(url, **kwargs):
+        return requests.get(url, **kwargs)
+
+    def _connect(self, **kwargs):
+        self.status_code = 'Unknown'
+        self.timeout = kwargs.get('timeout', 5.0)
+        self.proxies = kwargs.get('proxies', '')
+        try:
+            r = self.rate_limited_get(
+                self.url, 
+                params=self.params, 
+                headers=self.headers, 
+                timeout=self.timeout,
+                proxies=self.proxies
+            )
+            self.status_code = r.status_code
+            self.url = r.url
+            self.content = r.json()
+        except KeyboardInterrupt:
+            sys.exit()
+        except:
+            self.status_code = 404
+            self.error = 'ERROR - URL Connection'
+
+        # Open JSON content from Request connection
+        if self.status_code == 200:
+            try:
+                self.content = r.json()
+            except:
+                self.status_code = 400
+                self.error = 'ERROR - JSON Corrupted'
+                self.content = r.content
+
+    def _initialize(self, **kwargs):
+        self._connect(url=self.url, params=self.params, **kwargs)
+        self._parse(self.content)
+        self._json()
+        self.bbox
 
     def debug(self):
         print('# Debug')
@@ -31,11 +82,17 @@ class Base(object):
         print('* Status Code: {0}'.format(self.status_code))
         for key, value in self.params.items():
             print('* Parameter [{0}]: {1}'.format(key, value))
+        for key, value in self.headers.items():
+            print('* Headers [{0}]: {1}'.format(key, value))
         print('')
         print('## JSON Attributes')
         for key, value in self.json.items():
             print('* {0}: {1}'.format(key, value))
         print('')
+        print('## OSM Attributes')
+        for key, value in self.osm.items():
+            print('* {0}: {1}'.format(key, value))
+        print('')
         print('## Provider\'s Attributes')
         if self.parse:
             for key, value in self.parse.items():
@@ -48,71 +105,14 @@ class Base(object):
         else:
             print(self.content)
 
-    def help(self):
-        print('# {0}'.format(self.provider.title()))
-        print('')
-        print(self._description)
-        print('Using Geocoder you can retrieve {0}\'s geocoded data from {1}.'.format(self.provider, self.api))
-        print('')
-        print('## Python Example')
-        print('')
-        print('```python')
-        print('>>> import geocoder # pip install geocoder')
-        if self._example:
-            for line in self._example:
-                print(line)
-        else:
-            print('>>> g = geocoder.{0}(\'<address>\')'.format(self.provider.lower()))
-            print('>>> g.lat, g.lng')
-            print('45.413140 -75.656703')
-        print('...')
-        print('```')
-        print('')
-        print('## Geocoder Attributes')
-        print('')
-        for attribute in self.attributes:
-            print('* {0}'.format(attribute))
-        print('')
-        print('## Parameters')
-        print('')
-        for parameter in self._base_parameter + self._api_parameter:
-            print('* {0}'.format(parameter))
-        print('')
-
-        print('## References')
-        print('')
-        for reference in self._base_reference + self._api_reference:
-            print('* {0}'.format(reference))
-        print('')
-
     def _json(self):
         for key in dir(self):
             if bool(not key.startswith('_') and key not in self._exclude):
-                self.attributes.append(key)
+                self._attributes.append(key)
                 value = getattr(self, key)
                 if value:
                     self.json[key] = value
 
-    def _connect(self):
-        self.content = None
-        self.status_code = 404
-        try:
-            r = requests.get(self.url, params=self.params, headers=self.headers, timeout=self._timeout)
-            self.status_code = r.status_code
-            self.url = r.url
-        except KeyboardInterrupt:
-            sys.exit()
-        except:
-            self.error = 'ERROR - URL Connection'
-
-        # Open JSON content from Request connection
-        if self.status_code == 200:
-            try:
-                self.content = r.json()
-            except:
-                self.error = 'ERROR - JSON Corrupted'
-                self.content = r.content
-
     def _parse(self, content, last=''):
         # DICTIONARY
         if isinstance(content, dict):
@@ -247,11 +247,16 @@ class Base(object):
         self.southeast = [south, east]
 
         if bool(south and east and north and west):
-            bbox = dict()
-            bbox['northeast'] = [north, east]
-            bbox['southwest'] = [south, west]
+            bbox = {
+                'northeast': [north, east],
+                'southwest': [south, west],
+            }
             return bbox
-        return str('')
+        return {}
+
+    @property
+    def bbox(self):
+        return {}
 
     @property
     def ok(self):
@@ -263,11 +268,32 @@ class Base(object):
     @property
     def geometry(self):
         if self.ok:
-            geometry = dict()
-            geometry['type'] = 'Point'
-            geometry['coordinates'] = [self.lng, self.lat]
+            geometry = {
+                'type': 'Point',
+                'coordinates': [self.lng, self.lat],
+            }
             return geometry
-        return dict()
+        return {}
+
+    @property
+    def osm(self):
+        osm = dict()
+        if self.ok:
+            if self.housenumber:
+                osm['addr:housenumber'] = self.housenumber
+            if self.street:
+                osm['addr:street'] = self.street
+            if self.city:
+                osm['addr:city'] = self.city
+            if self.state:
+                osm['addr:state'] = self.state
+            if self.country:
+                osm['addr:country'] = self.country
+            if self.postal:
+                osm['addr:postal'] = self.postal
+            if self.population:
+                osm['population'] = self.population
+        return osm
 
     @property
     def wkt(self):
@@ -277,7 +303,9 @@ class Base(object):
 
     @property
     def latlng(self):
-        return [self.lat, self.lng]
+        if self.ok:
+            return [self.lat, self.lng]
+        return []
 
     @property
     def y(self):
@@ -286,3 +314,20 @@ class Base(object):
     @property
     def x(self):
         return self.lng
+
+    @property
+    def locality(self):
+        return self.city
+
+    @property
+    def province(self):
+        return self.state
+
+    @property
+    def street_number(self):
+        return self.housenumber
+
+    @property
+    def route(self):
+        return self.street
+
